#!/usr/bin/env python
# -*- coding: utf-8 -*-

import random
import re
import string

class Markov(object):
    
    def __init__(self, chunks):
        self.cache = {}
        self.chunks = chunks
        self.words = self.chunks_to_words()
        self.word_size = len(self.words)
        self.out_size = len(self.words)/len(self.chunks)
        self.make_cache()
       
    
    def chunks_to_words(self):
        words = []
        for chunk in self.chunks:
            chunk.text = re.sub(r'<[^>]*?>', '', chunk.text) 
            words.extend(chunk.text.split())
        return words
    
    def as_key(self, word):
        return string.lower(word)
    
    def triples(self):
        """ Generates triples from the given data string. So if our string were
           "What a lovely day", we'd generate (What, a, lovely) and then
           (a, lovely, day).
        """
       
        if len(self.words) < 3:
            return
       
        for i in range(len(self.words) - 2):
            yield (self.words[i], self.words[i+1], self.words[i+2])
         
    def make_cache(self):
        for w1, w2, w3 in self.triples():
            key = (self.as_key(w1), self.as_key(w2))
            if key in self.cache:
                self.cache[key].append(w3)
            else:
                self.cache[key] = [w3]
        
    # ${\small \textnormal{\useTextGlyph{fxl}{summation}}}$
    def generate_markov_text(self, length = False):
        if length == False:
            length = self.out_size
            
        seed = random.randint(0, self.word_size-3)
        w1 = self.words[seed] 
        w2 = self.words[seed+1]
        gen_words = []
        for i in xrange(length):
            gen_words.append(w1)
            w1, w2 = w2, random.choice(self.cache[(self.as_key(w1), self.as_key(w2))])
        gen_words.append(w2)
        return ' '.join(gen_words)
         
if __name__ == '__main__':
    print 'Oh, no...'
